/** \file   my_hash.h
 *  \brief  A simple hash table
 *  \author Takahisa Toda
 */
#ifndef MY_HASH_H
#define MY_HASH_H

#define HASH_DEFAULT_SIZE        (16777216UL  + 43)     //!< 2^{24} + 43 
//#define HASH_DEFAULT_SIZE        (1UL << 20)

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/** \brief a linked list for a hash table*/
struct htable_entry {
  size_t key;
  size_t data;
  struct htable_entry *next;
};

/** \brief hash table (chaining)*/
struct htable {
  size_t nbucket; //!< the number of buckets
  struct htable_entry **bucket; //!< the i-th bucket: bucket[i]
};


/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

extern struct htable *ht_create(size_t nbuckets);
extern void ht_destroy(struct htable *h);
extern void ht_add(size_t key, size_t data, struct htable *h);
extern int ht_search(size_t key, size_t *pdata, const struct htable *h);
extern size_t ht_maxdepth(const struct htable *h);
extern void ht_enlarge(struct htable *h);

#endif /*MY_HASH_H*/
